/*
 * $Id$
 */

package org.apache.maven.plugin.tools.model.io.xpp3;

  //---------------------------------/
 //- Imported classes and packages -/
//---------------------------------/

import java.io.Writer;
import java.util.Iterator;
import org.apache.maven.plugin.tools.model.Component;
import org.apache.maven.plugin.tools.model.LifecycleExecution;
import org.apache.maven.plugin.tools.model.Mojo;
import org.apache.maven.plugin.tools.model.Parameter;
import org.apache.maven.plugin.tools.model.PluginMetadata;
import org.codehaus.plexus.util.xml.pull.*;

/**
 * Class PluginMetadataXpp3Writer.
 * 
 * @version $Revision$ $Date$
 */
public class PluginMetadataXpp3Writer {


      //--------------------------/
     //- Class/Member Variables -/
    //--------------------------/

    /**
     * Field serializer
     */
    private org.codehaus.plexus.util.xml.pull.XmlSerializer serializer;

    /**
     * Field NAMESPACE
     */
    private String NAMESPACE;


      //-----------/
     //- Methods -/
    //-----------/

    /**
     * Method write
     * 
     * @param writer
     * @param pluginMetadata
     */
    public void write(Writer writer, PluginMetadata pluginMetadata)
        throws java.io.IOException
    {
        serializer = new MXSerializer();
        serializer.setProperty( "http://xmlpull.org/v1/doc/properties.html#serializer-indentation", "  " );
        serializer.setProperty( "http://xmlpull.org/v1/doc/properties.html#serializer-line-separator", "\n" );
        serializer.setOutput( writer );
        serializer.startDocument( pluginMetadata.getModelEncoding(), null );
        writePluginMetadata( pluginMetadata, "pluginMetadata", serializer );
        serializer.endDocument();
    } //-- void write(Writer, PluginMetadata) 

    /**
     * Method writeComponent
     * 
     * @param component
     * @param serializer
     * @param tagName
     */
    private void writeComponent(Component component, String tagName, XmlSerializer serializer)
        throws java.io.IOException
    {
        if ( component != null )
        {
            serializer.startTag( NAMESPACE, tagName );
            if ( component.getRole() != null )
            {
                serializer.startTag( NAMESPACE, "role" ).text( component.getRole() ).endTag( NAMESPACE, "role" );
            }
            if ( component.getHint() != null )
            {
                serializer.startTag( NAMESPACE, "hint" ).text( component.getHint() ).endTag( NAMESPACE, "hint" );
            }
            serializer.endTag( NAMESPACE, tagName );
        }
    } //-- void writeComponent(Component, String, XmlSerializer) 

    /**
     * Method writeLifecycleExecution
     * 
     * @param lifecycleExecution
     * @param serializer
     * @param tagName
     */
    private void writeLifecycleExecution(LifecycleExecution lifecycleExecution, String tagName, XmlSerializer serializer)
        throws java.io.IOException
    {
        if ( lifecycleExecution != null )
        {
            serializer.startTag( NAMESPACE, tagName );
            if ( lifecycleExecution.getLifecycle() != null )
            {
                serializer.startTag( NAMESPACE, "lifecycle" ).text( lifecycleExecution.getLifecycle() ).endTag( NAMESPACE, "lifecycle" );
            }
            if ( lifecycleExecution.getPhase() != null )
            {
                serializer.startTag( NAMESPACE, "phase" ).text( lifecycleExecution.getPhase() ).endTag( NAMESPACE, "phase" );
            }
            if ( lifecycleExecution.getGoal() != null )
            {
                serializer.startTag( NAMESPACE, "goal" ).text( lifecycleExecution.getGoal() ).endTag( NAMESPACE, "goal" );
            }
            serializer.endTag( NAMESPACE, tagName );
        }
    } //-- void writeLifecycleExecution(LifecycleExecution, String, XmlSerializer) 

    /**
     * Method writeMojo
     * 
     * @param mojo
     * @param serializer
     * @param tagName
     */
    private void writeMojo(Mojo mojo, String tagName, XmlSerializer serializer)
        throws java.io.IOException
    {
        if ( mojo != null )
        {
            serializer.startTag( NAMESPACE, tagName );
            if ( mojo.getGoal() != null )
            {
                serializer.startTag( NAMESPACE, "goal" ).text( mojo.getGoal() ).endTag( NAMESPACE, "goal" );
            }
            if ( mojo.getPhase() != null )
            {
                serializer.startTag( NAMESPACE, "phase" ).text( mojo.getPhase() ).endTag( NAMESPACE, "phase" );
            }
            if ( mojo.isAggregator() != false )
            {
                serializer.startTag( NAMESPACE, "aggregator" ).text( String.valueOf( mojo.isAggregator() ) ).endTag( NAMESPACE, "aggregator" );
            }
            if ( mojo.getRequiresDependencyResolution() != null )
            {
                serializer.startTag( NAMESPACE, "requiresDependencyResolution" ).text( mojo.getRequiresDependencyResolution() ).endTag( NAMESPACE, "requiresDependencyResolution" );
            }
            if ( mojo.isRequiresProject() != false )
            {
                serializer.startTag( NAMESPACE, "requiresProject" ).text( String.valueOf( mojo.isRequiresProject() ) ).endTag( NAMESPACE, "requiresProject" );
            }
            if ( mojo.isRequiresReports() != false )
            {
                serializer.startTag( NAMESPACE, "requiresReports" ).text( String.valueOf( mojo.isRequiresReports() ) ).endTag( NAMESPACE, "requiresReports" );
            }
            if ( mojo.isRequiresOnline() != false )
            {
                serializer.startTag( NAMESPACE, "requiresOnline" ).text( String.valueOf( mojo.isRequiresOnline() ) ).endTag( NAMESPACE, "requiresOnline" );
            }
            if ( mojo.isInheritByDefault() != false )
            {
                serializer.startTag( NAMESPACE, "inheritByDefault" ).text( String.valueOf( mojo.isInheritByDefault() ) ).endTag( NAMESPACE, "inheritByDefault" );
            }
            if ( mojo.isRequiresDirectInvocation() != false )
            {
                serializer.startTag( NAMESPACE, "requiresDirectInvocation" ).text( String.valueOf( mojo.isRequiresDirectInvocation() ) ).endTag( NAMESPACE, "requiresDirectInvocation" );
            }
            if ( mojo.getExecution() != null )
            {
                writeLifecycleExecution( mojo.getExecution(), "execution", serializer );
            }
            if ( mojo.getComponents() != null && mojo.getComponents().size() > 0 )
            {
                serializer.startTag( NAMESPACE, "components" );
                for ( Iterator iter = mojo.getComponents().iterator(); iter.hasNext(); )
                {
                    Component o = (Component) iter.next();
                    writeComponent( o, "component", serializer );
                }
                serializer.endTag( NAMESPACE, "components" );
            }
            if ( mojo.getParameters() != null && mojo.getParameters().size() > 0 )
            {
                serializer.startTag( NAMESPACE, "parameters" );
                for ( Iterator iter = mojo.getParameters().iterator(); iter.hasNext(); )
                {
                    Parameter o = (Parameter) iter.next();
                    writeParameter( o, "parameter", serializer );
                }
                serializer.endTag( NAMESPACE, "parameters" );
            }
            if ( mojo.getDescription() != null )
            {
                serializer.startTag( NAMESPACE, "description" ).text( mojo.getDescription() ).endTag( NAMESPACE, "description" );
            }
            if ( mojo.getDeprecation() != null )
            {
                serializer.startTag( NAMESPACE, "deprecated" ).text( mojo.getDeprecation() ).endTag( NAMESPACE, "deprecated" );
            }
            if ( mojo.getCall() != null )
            {
                serializer.startTag( NAMESPACE, "call" ).text( mojo.getCall() ).endTag( NAMESPACE, "call" );
            }
            serializer.endTag( NAMESPACE, tagName );
        }
    } //-- void writeMojo(Mojo, String, XmlSerializer) 

    /**
     * Method writeParameter
     * 
     * @param parameter
     * @param serializer
     * @param tagName
     */
    private void writeParameter(Parameter parameter, String tagName, XmlSerializer serializer)
        throws java.io.IOException
    {
        if ( parameter != null )
        {
            serializer.startTag( NAMESPACE, tagName );
            if ( parameter.getName() != null )
            {
                serializer.startTag( NAMESPACE, "name" ).text( parameter.getName() ).endTag( NAMESPACE, "name" );
            }
            if ( parameter.getAlias() != null )
            {
                serializer.startTag( NAMESPACE, "alias" ).text( parameter.getAlias() ).endTag( NAMESPACE, "alias" );
            }
            if ( parameter.getProperty() != null )
            {
                serializer.startTag( NAMESPACE, "property" ).text( parameter.getProperty() ).endTag( NAMESPACE, "property" );
            }
            if ( parameter.isRequired() != false )
            {
                serializer.startTag( NAMESPACE, "required" ).text( String.valueOf( parameter.isRequired() ) ).endTag( NAMESPACE, "required" );
            }
            if ( parameter.isReadonly() != false )
            {
                serializer.startTag( NAMESPACE, "readonly" ).text( String.valueOf( parameter.isReadonly() ) ).endTag( NAMESPACE, "readonly" );
            }
            if ( parameter.getExpression() != null )
            {
                serializer.startTag( NAMESPACE, "expression" ).text( parameter.getExpression() ).endTag( NAMESPACE, "expression" );
            }
            if ( parameter.getDefaultValue() != null )
            {
                serializer.startTag( NAMESPACE, "defaultValue" ).text( parameter.getDefaultValue() ).endTag( NAMESPACE, "defaultValue" );
            }
            if ( parameter.getType() != null )
            {
                serializer.startTag( NAMESPACE, "type" ).text( parameter.getType() ).endTag( NAMESPACE, "type" );
            }
            if ( parameter.getDescription() != null )
            {
                serializer.startTag( NAMESPACE, "description" ).text( parameter.getDescription() ).endTag( NAMESPACE, "description" );
            }
            if ( parameter.getDeprecation() != null )
            {
                serializer.startTag( NAMESPACE, "deprecated" ).text( parameter.getDeprecation() ).endTag( NAMESPACE, "deprecated" );
            }
            serializer.endTag( NAMESPACE, tagName );
        }
    } //-- void writeParameter(Parameter, String, XmlSerializer) 

    /**
     * Method writePluginMetadata
     * 
     * @param pluginMetadata
     * @param serializer
     * @param tagName
     */
    private void writePluginMetadata(PluginMetadata pluginMetadata, String tagName, XmlSerializer serializer)
        throws java.io.IOException
    {
        if ( pluginMetadata != null )
        {
            serializer.startTag( NAMESPACE, tagName );
            if ( pluginMetadata.getMojos() != null && pluginMetadata.getMojos().size() > 0 )
            {
                serializer.startTag( NAMESPACE, "mojos" );
                for ( Iterator iter = pluginMetadata.getMojos().iterator(); iter.hasNext(); )
                {
                    Mojo o = (Mojo) iter.next();
                    writeMojo( o, "mojo", serializer );
                }
                serializer.endTag( NAMESPACE, "mojos" );
            }
            serializer.endTag( NAMESPACE, tagName );
        }
    } //-- void writePluginMetadata(PluginMetadata, String, XmlSerializer) 

}
